import template from './orders-detail.component.html';
import { usStates } from '../../../common/usStates';
import { countries } from '../../../common/countries';
declare const angular: ng.IAngularStatic;

export const OrdersDetailComponent: ng.IComponentOptions = {
	controller: OrderController,
	template,
};

OrderController.$inject = [
	'$http',
	'$scope',
	'$state',
	'$stateParams',
	'$uibModal',
	'Admin',
	'Debug',
	'Localized',
	'Utils',
];

function OrderController(
	$http: ng.IHttpService,
	$scope: ng.IScope,
	$state: ng.ui.IStateService,
	$stateParams: ng.ui.IStateParamsService,
	$uibModal: ng.ui.bootstrap.IModalService,
	Admin: any,
	Debug: any,
	Localized: any,
	Utils: any,
) {
	const vm = this;
	this.isBodyCopied = [];
	this.isResponseCopied = [];
	vm.downloadPurchaseOrder = downloadPurchaseOrder;
	vm.exportToQuickbooks = exportToQuickbooks;
	vm.generatePurchaseOrders = generatePurchaseOrders;
	vm.lockCosts = lockCosts;
	vm.openRmaModal = openRmaModal;
	vm.openShipmentModal = openShipmentModal;
	vm.resendEmail = resendEmail;

	/**
	 * Initialization
	 */
	this.$onInit = () => {
		vm.id = $stateParams.id || null;
		vm.isLoading = true;
		vm.selectedOrder = {};

		this.breadcrumbs = [
			{ name: 'Orders', state: 'orders.index' },
			{ name: `Order ${ vm.id }`, state: `order({id:${ vm.id }})` },
		];

		if ( vm.id ) {
			this.getOrder();
		} else {
			$state.go( 'orders.index' );
		}
	};

	this.copyToClipboard = ( content: string, type: 'body'|'response', index: number, e: Event ) => {
		e.preventDefault();
		navigator.clipboard.writeText( content )
			.then( () => {
				$scope.$apply( () => {
					this.isBodyCopied.fill( false );
					this.isResponseCopied.fill( false );
					if ( type === 'body' ) {
						this.isBodyCopied[ index ] = true;
					} else {
						this.isResponseCopied[ index ] = true;
					}
				} );
			}, () => {
				console.error( 'Could not copy to clipboard' );
			} );
	};

	this.includesCosts = ( order ) => {
		if ( order.lineItems ) {
			return order.lineItems.some( ( item ) => {
				return item.costs ? item.costs.length : false;
			} );
		}
		return false;
	};

	this.includesNotificationDetails = ( order ) => {
		if ( order.order_notifications ) {
			return order.order_notifications.some( ( item ) => ( item.response || item.body ) );
		}
		return false;
	};

	this.includesPurchaseOrders = ( order ) => {
		if ( order.lineItems ) {
			return order.lineItems.some( ( item ) => item.purchase_order );
		}
		return false;
	};

	/**
	 */
	this.getOrder = () => {
		vm.isLoading = true;
		const endpoint = `${ Localized.apiURL }/admin/orders/${ vm.id }`;

		Utils.httpGet( endpoint )
			.then( ( response: any ) => {
				vm.selectedOrder = response.order;
				this.hasCosts = this.includesCosts( response.order );
				this.hasPurchaseOrders = this.includesPurchaseOrders( response.order );
				this.hasNotificationDetails = this.includesNotificationDetails( response.order );
				vm.isLoading = false;
			} );
	};

	this.getLineItemStatus = ( item: { status: string, 'purchase_order': { status: string } } ): string => {
		const allowList = [ 'accept', 'reject', 'backordered', 'sent' ];
		if ( item.purchase_order?.status && allowList.includes( item.purchase_order.status ) ) {
			return item.purchase_order.status;
		}
		return item.status;
	};

	this.getNonUnderscored = ( str: string ): string => {
		return str.replace( /_/g, ' ' );
	};

	this.goToProduct = ( item: any, e: Event ) => {
		if ( e ) {
			e.preventDefault();
		}
		return $state.go( 'catalog-products', { activeTab: 2, product: item } );
	};

	/**
	 * @param {Object} item Product
	 */
	function openRmaModal( item: any ) {
		const order = vm.selectedOrder;
		$uibModal.open( {
			templateUrl: Localized.root.url + '/assets/app/modals/rma.html',
			size: 'lg',
			// eslint-disable-next-line no-shadow
			controller( $scope: any, $uibModalInstance: ng.ui.bootstrap.IModalInstanceService ) {
				$scope.order = order;
				$scope.item = item;
				$scope.returnStatuses = {
					none: 'No return requested',
					requested: 'Requested',
					denied: 'Denied',
					approved: 'Approved',
					refunded: 'Refunded',
					replaced: 'Replaced',
				};
				$scope.returnReasons = {
					defective: 'Defective',
					'wrong item': 'Wrong Item',
					exchange: 'Exchange',
					other: 'Other',
				};
				$scope.Confirm = function() {
					vm.isLoading = true;
					const req = {
						method: 'PUT',
						url: Localized.apiURL + '/admin/rma/' + $scope.item.purch_id,
						data: $scope.item,
					};
					Utils.getHttpPromise( req ).then( function() {
						$uibModalInstance.close();
					}, function( errResp: Error ) {
						Debug.error( errResp );
					} ).finally( function() {
						vm.isLoading = false;
					} );
				};
				$scope.Cancel = function() {
					$uibModalInstance.dismiss();
				};
			},
		} ).result.then( () => {
			this.getOrder();
		} );
	}

	/**
	 */
	function openShipmentModal() {
		const id = vm.id;
		$uibModal.open( {
			templateUrl: Localized.root.url + '/assets/app/modals/shipment.html',
			size: 'lg',
			// eslint-disable-next-line no-shadow
			controller( $scope: any, $uibModalInstance: ng.ui.bootstrap.IModalInstanceService ) {
				$scope.isLoading = false;
				$scope.shipment = {
					order_id: id,
					country_code: 'US',
					country_phone_code: 1,
				};
				$scope.states = usStates;
				$scope.countries = countries;
				$scope.Confirm = function() {
					vm.isLoading = true;
					const req = {
						method: 'POST',
						url: Localized.apiURL + '/admin/order/shipment/add',
						data: $scope.shipment,
					};
					Utils.getHttpPromise( req ).then( function() {
						$uibModalInstance.close();
					}, function( errResp: Error ) {
						Debug.error( errResp );
						$scope.isLoading = false;
					} );
				};
				$scope.Cancel = function() {
					$uibModalInstance.dismiss();
				};
			},
		} ).result.then( () => {
			this.getOrder();
		} );
	}

	/**
	 */
	this.resendToGP = () => {
		this.isResendingToGP = true;
		const req = {
			method: 'POST',
			url: Localized.apiURL + '/admin/order/resend',
			data: {
				order_id: vm.id,
			},
		};
		Utils.getHttpPromise( req )
			.then( ( response ) => {
				if ( ! response.data.errors.length && ! response.data.warnings.length ) {
					this.getOrder();
				}
			} )
			.catch( ( errResp: Error ) => {
				Debug.error( errResp );
			} )
			.finally( () => {
				this.isResendingToGP = false;
			} );
	};

	/**
	 * Attempt to export to quickbooks
	 */
	function exportToQuickbooks() {
		this.isExportingToQuickbooks = true;
		const req = {
			method: 'POST',
			url: Localized.apiURL + '/admin/order/' + vm.id + '/quickbooks/export',
		};
		Utils.getHttpPromise( req )
			.then( () => {
				angular.noop();
			} )
			.catch( ( errResp: Error ) => {
				Debug.error( errResp );
			} )
			.finally( () => {
				this.isExportingToQuickbooks = false;
			} );
	}

	/**
	 */
	function resendEmail() {
		this.isResendingEmail = true;
		const req = {
			method: 'POST',
			url: Localized.apiURL + '/admin/order/resend-email',
			data: {
				order_id: vm.id,
			},
		};
		Utils.getHttpPromise( req )
			.then( () => {
				angular.noop();
			} )
			.catch( ( errResp: Error ) => {
				Debug.error( errResp );
			} )
			.finally( () => {
				this.isResendingEmail = false;
			} );
	}

	this.resendNotification = () => {
		this.isResendingNotification = true;
		const url = `${ localized.apiURL }/admin/order/resend-email`;
		const data = {
			order_id: vm.id,
			notifications: true,
		};

		$http.post( url, data )
			.then( ( response ) => {
				Localized.setNotifications( response.data );
				this.isResendingNotification = false;
			} );
	};

	/**
	 */
	function lockCosts() {
		vm.isLoading = true;
		const req = {
			method: 'POST',
			url: Localized.apiURL + '/admin/order/' + vm.id + '/costs/lock',
			data: {
				costs: vm.selectedOrder.purchased_items,
			},
		};
		Utils.getHttpPromise( req )
			.then( function() {
				angular.noop();
			}, function( errResp: Error ) {
				Debug.log( errResp );
			} ).finally( function() {
				vm.isLoading = false;
			} );
	}

	/**
	 */
	function generatePurchaseOrders() {
		vm.isLoading = true;
		const req = {
			method: 'POST',
			url: Localized.apiURL + '/admin/order/' + vm.id + '/po/generate',
		};
		Utils.getHttpPromise( req )
			.then( function() {
				angular.noop();
			}, function( errResp: Error ) {
				Debug.log( errResp );
			} ).finally( function() {
				vm.isLoading = false;
			} );
	}

	/**
	 * @param {Object} purchaseOrder Purchase Order
	 */
	this.sendPurchaseOrder = ( purchaseOrder: any ) => {
		vm.isLoading = true;
		const req = {
			method: 'POST',
			url: Localized.apiURL + '/admin/order/' + purchaseOrder.order_id + '/po/' + purchaseOrder.id + '/transmit',
		};
		Utils.getHttpPromise( req )
			.then( ( resp: any ) => {
				$scope.order = resp.order;
			}, ( errResp: Error ) => {
				Debug.log( errResp );
			} ).finally( () => {
				vm.isLoading = false;
			} );
	};

	/**
	 * @param {Object} purchaseOrder Purchase Order
	 */
	function downloadPurchaseOrder( purchaseOrder: any ) {
		const url = `${ Localized.apiURL }/admin/order/${ purchaseOrder.order_id }/po/${ purchaseOrder.id }/download`;

		window.open( Admin.Download( url ), '_blank' );
	}
}
