import React, { SyntheticEvent, useEffect, useState } from 'react';
import {
	Button,
	Card,
	CardBody,
	CardHeader,
	Modal,
	RadioControl,
	SelectControl,
	TextControl,
} from '@wordpress/components';
import { Breadcrumb } from '../../shared/Breadcrumb';
import { Notices } from '../../shared/Notices';
import { vfAPI } from '../../core/vfAPI';
import { stateOptions } from '../../shared/stateOptions';
import { PasswordControl } from '../../shared/PasswordControl';

interface IProps {
	profileId: number;
	isNew: boolean;
}

interface IProfile {
	phone?: string | number;
	zip?: string;
	state?: string|null;
	city?: string;
	address1?: string;
	address2?: string;
	address3?: string;
	'receiver_id_type'?: string;
	'receiver_id'?: string | number;
	'account_user'?: string;
	'account_password'?: string;
	endpoint?: string;
	format?: 'cxml'|'pdf'|'csv'|null;
    id?: number;
    name?: string;
    email?: string;
}

const apiUrl = '/purchasing/document-profile/';
const formatOptions = [
	{ label: 'cXML', value: 'cxml' },
	{ label: 'PDF', value: 'pdf' },
	{ label: 'CSV', value: 'csv' },
];
const receiverOptions = [
	{ label: 'Network ID', value: 'NetworkID' },
	{ label: 'DUNS', value: 'DUNS' },
];

export const DocumentProfile = ( { isNew, profileId }: IProps ) => {
	const [ breadcrumbs, setBreadcrumbs ] = useState( [
		{ name: 'Purchasing', hash: '#!/purchasing' },
		{ name: 'Document Profiles', hash: '#!/purchasing/document-profiles' },
	] );
	const [ isBusy, setIsBusy ] = useState<boolean>( true );
	const [ isOpen, setOpen ] = useState( false );
	const [ notices, setNotices ] = useState( [] );
	const [ profile, setProfile ] = useState<IProfile>( {} );

	useEffect( () => {
		if ( profileId ) {
			getProfile( profileId );
		}
	}, [ profileId ] );

	useEffect( () => {
		if ( isNew ) {
			setBreadcrumbs( [ ...breadcrumbs, { name: 'Add New', hash: '#!/purchasing/document-profiles/new' } ] );
			setIsBusy( false );
		}
	}, [ isNew ] );

	useEffect( () => {
		if ( isNew && profile.id ) {
			location.assign( location.href.replace( 'new', profile.id.toString() ) );
		}
	}, [ profile ] );

	const closeModal = () => setOpen( false );

	const deleteProfile = () => {
		setIsBusy( true );
		vfAPI.delete( `${ apiUrl }${ profile.id }` )
			.then( ( response ) => {
				setNotices( response.data.notices );
				if ( ! response.data.errors.length ) {
					location.assign( location.href.replace( `/${ profile.id.toString() }`, '' ) );
				}
			} );
	};

	const getProfile = ( id: number ) => {
		setIsBusy( true );
		vfAPI.get( `${ apiUrl }${ id }` )
			.then( ( response ) => {
				setNotices( response.data.notices );
				if ( ! response.data.errors.length ) {
					setBreadcrumbs( [ ...breadcrumbs, {
						name: response.data.document_profile.name,
						hash: `#!/purchasing/document-profiles/${ profileId }` } ] );
					setProfile( response.data.document_profile );
				}
				setIsBusy( false );
			} );
	};

	const handleSubmit = ( e: SyntheticEvent ) => {
		e.preventDefault();
		if ( isNew ) {
			save();
		} else {
			update();
		}
	};

	const openModal = () => setOpen( true );

	const save = () => {
		setIsBusy( true );
		vfAPI.post( apiUrl, profile )
			.then( ( response ) => {
				setNotices( response.data.notices );
				if ( ! response.data.errors.length ) {
					setProfile( response.data.document_profile );
				}
				setIsBusy( false );
			} );
	};

	const update = () => {
		setIsBusy( true );
		vfAPI.put( `${ apiUrl }${ profile.id }`, profile )
			.then( ( response ) => {
				setNotices( response.data.notices );
				if ( ! response.data.errors.length ) {
					setProfile( response.data.document_profile );
				}
				setIsBusy( false );
			} );
	};
	return (
		<>
			<Breadcrumb breadcrumbs={ breadcrumbs } />
			<Notices notices={ notices } />
			<h2>{ isNew ? 'Add New' : 'Edit' } Document Profile</h2>
			<form onSubmit={ handleSubmit }>
				<fieldset disabled={ isBusy }>
					<div className="row mb-3">
						<div className="col-md-5">
							<TextControl label="Name (required)" value={ profile.name } onChange={ ( value ) => setProfile( { ...profile, name: value } ) } required />
							<SelectControl label="Format (required)" value={ profile.format } onChange={ ( value ) => setProfile( { ...profile, format: value } ) } options={ [ { label: '', value: null }, ...formatOptions ] } required />
							<TextControl label="Endpoint" type="url" value={ profile.endpoint } onChange={ ( value ) => setProfile( { ...profile, endpoint: value } ) } />
							<TextControl label="Account User" value={ profile.account_user } onChange={ ( value ) => setProfile( { ...profile, account_user: value } ) } />
							<PasswordControl label="Account Password" value={ profile.account_password } onChange={ ( value ) => setProfile( { ...profile, account_password: value } ) } />
							<TextControl label="Receiver ID" value={ profile.receiver_id } onChange={ ( value ) => setProfile( { ...profile, receiver_id: value } ) } />
							<RadioControl label="Receiver ID Type" selected={ profile.receiver_id_type } onChange={ ( value ) => setProfile( { ...profile, receiver_id_type: value } ) } options={ receiverOptions } />
						</div>
						<div className="col-md-7">
							<fieldset>
								<Card>
									<CardHeader>
										Account Receivable
									</CardHeader>
									<CardBody>
										<div className="row">
											<div className="col-sm">
												<TextControl label="Email (required)" type="email" value={ profile.email } onChange={ ( value ) => setProfile( { ...profile, email: value } ) } required />
											</div>
											<div className="col-sm">
												<TextControl label="Phone" value={ profile.phone } onChange={ ( value ) => setProfile( { ...profile, phone: value } ) } />
											</div>
										</div>
										<TextControl label="Street Address" value={ profile.address1 } onChange={ ( value ) => setProfile( { ...profile, address1: value } ) } />
										<TextControl label="Apt., Suite, etc." value={ profile.address2 } onChange={ ( value ) => setProfile( { ...profile, address2: value } ) } />
										<TextControl label="Additional Address" value={ profile.address3 } onChange={ ( value ) => setProfile( { ...profile, address3: value } ) } />
										<div className="row align-items-baseline">
											<div className="col-sm">
												<TextControl label="City" value={ profile.city } onChange={ ( value ) => setProfile( { ...profile, city: value } ) } />
											</div>
											<div className="col-sm">
												<SelectControl label="State" value={ profile.state } onChange={ ( value ) => setProfile( { ...profile, state: value } ) } options={ [ { label: '', value: null }, ...stateOptions ] } />
											</div>
											<div className="col-sm">
												<TextControl label="Zip Code" value={ profile.zip } onChange={ ( value ) => setProfile( { ...profile, zip: value } ) } />
											</div>
										</div>
									</CardBody>
								</Card>
							</fieldset>
						</div>
					</div>
				</fieldset>
				<div className="hstack gap-1">
					<Button isBusy={ isBusy } variant="primary" type="submit">{ isNew ? 'Add New' : 'Update' } Document Profile</Button>
					{ profile.id &&
					<Button isBusy={ isBusy } isDestructive onClick={ openModal }>Delete</Button>
					}
				</div>
			</form>
			{ isOpen && (
				<Modal title="Delete this document profile?" onRequestClose={ closeModal }>
					<p>This will delete this document profile.</p>
					<div className="hstack gap-1 justify-content-end">
						<Button variant="secondary" onClick={ closeModal }>
							Cancel
						</Button>
						<Button variant="primary" isDestructive onClick={ deleteProfile }>
							Delete
						</Button>
					</div>
				</Modal>
			) }
		</>
	);
};
