import React, { ReactElement, useEffect, useState } from 'react';
import { Card, CardBody, CardHeader, Spinner } from '@wordpress/components';
import { vfAPI } from '../../admin/core/vfAPI';

const apiUrl = '/stats/dashboard/';

interface Orders {
    today: number;
    'today_totals': number;
    last7: number;
    'last7_totals': number;
    prev7: number;
    'prev7_totals': number;
    last30: number;
    'last30_totals': number;
    prev30: number;
    'prev30_totals': number;
    weeklygrowth: number;
    monthlygrowth: number;
}

export const DashboardOrders = () => {
	const [ isBusy, setIsBusy ] = useState( true );
	const [ orders, setOrders ] = useState<Orders>();

	const toCurrency = ( num: number ): string => {
		return num.toLocaleString( 'en-US', { style: 'currency', currency: 'USD' } );
	};

	const toGrowth = ( last: number, prev: number ): ReactElement => {
		let iconClass = 'arrow-down-right';
		if ( last === prev ) {
			iconClass = 'arrow-right';
		} else if ( last > prev ) {
			iconClass = 'arrow-up-right';
		}

		let colorClass = 'text-muted';
		if ( last > prev ) {
			colorClass = 'text-success';
		} else if ( last < prev ) {
			colorClass = 'text-danger';
		}

		if ( last && prev ) {
			return <span className={ colorClass }><i className={ `bi bi-${ iconClass }` }></i>&nbsp;{ Math.round( ( ( last / prev ) - 1 ) * 100 ) }%</span>;
		}
	};

	const getOrders = () => {
		setIsBusy( true );
		vfAPI.get( apiUrl )
			.then( ( response ) => {
				if ( response.data?.orders ) {
					setOrders( response.data.orders );
				}
				setIsBusy( false );
			} );
	};

	useEffect( () => {
		getOrders();
	}, [] );

	return (
		<>
			{ isBusy &&
			<div className="hstack justify-content-center py-5"><Spinner /></div>
			}
			{ orders &&
				<>
					<h3 className="h5">Gross revenue</h3>
					<div className="row g-3 mb-4">
						<div className="col-md-4">
							<Card elevation={ 2 }>
								<CardHeader>
									Today
								</CardHeader>
								<CardBody>
									<div>
										<strong>
											<span className="fs-4">
												{ toCurrency( orders.today_totals ) }
											</span>
										</strong>
									</div>
									<small>{ orders.today ? orders.today.toLocaleString() : 'No' } orders</small>
								</CardBody>
							</Card>
						</div>
						<div className="col-md-4">
							<Card elevation={ 2 }>
								<CardHeader>
									Last 7 days
								</CardHeader>
								<CardBody>
									<div className="hstack justify-content-between">
										<strong>
											<span className="fs-4">
												{ toCurrency( orders.last7_totals ) }
											</span>
											{ orders.last7 > 0 &&
											<small> ({ orders.last7.toLocaleString() } order{ orders.last7 > 1 ? 's' : '' })</small>
											}
										</strong>
										<span>
											{ toGrowth( orders.last7_totals, orders.prev7_totals ) || <></> }
										</span>
									</div>
									<small>
										Previous 7 days: { toCurrency( orders.prev7_totals ) }
										{ orders.prev7 > 0 &&
											<small> ({ orders.prev7.toLocaleString() } order{ orders.prev7 > 1 ? 's' : '' })</small>
										}
									</small>
								</CardBody>
							</Card>
						</div>
						<div className="col-md-4">
							<Card elevation={ 2 }>
								<CardHeader>
									Last 30 days
								</CardHeader>
								<CardBody>
									<div className="hstack justify-content-between">
										<strong>
											<span className="fs-4">
												{ toCurrency( orders.last30_totals ) }
											</span>
											{ orders.last30 > 0 &&
											<small> ({ orders.last30.toLocaleString() } order{ orders.last30 > 1 ? 's' : '' })</small>
											}
										</strong>
										<span>
											{ toGrowth( orders.last30_totals, orders.prev30_totals ) }
										</span>
									</div>
									<small>
										Previous 30 days: { toCurrency( orders.prev30_totals ) }
										{ orders.prev30 > 0 &&
											<small> ({ orders.prev30.toLocaleString() } order{ orders.prev30 > 1 ? 's' : '' })</small>
										}
									</small>
								</CardBody>
							</Card>
						</div>
					</div>
				</>
			}
		</>
	);
};
