( function() {
	'use strict';

	angular
		.module( 'vendorfuelApp' )
		.factory( 'Utils', utilsFactory );

	utilsFactory.$inject = [
		'$http',
		'$location',
		'Admin',
		'Localized',
		'Tenant',
	];

	/**
	 * @param {Object} $http     Angular service
	 * @param {Object} $location Angular service
	 * @param {Object} Admin     VendorFuel service
	 * @param {Object} Localized VendorFuel service
	 * @param {Object} Tenant    VendorFuel service
	 * @return {Object} Utils factory
	 */
	function utilsFactory(
		$http: ng.IHttpService,
		$location: ng.ILocationService,
		Admin: any,
		Localized: any,
		Tenant: any,
	) {
		/**
		 * @param {Object} request HTTP request
		 * @return {Object} Promise
		 */
		function httpPromise( request: any ) {
			const promise = $http( request ).then(
				function successCallback( response: any ) {
					Localized.deleteNotifications();
					if ( response.data ) {
						if ( response.data.notifications ) {
							for ( let i = 0; i < response.data.notifications.length; i++ ) {
								Localized.setNotification( {
									type: 'success',
									message: response.data.notifications[ i ],
								} );
							}
						}
						if ( response.data.warnings ) {
							for ( let j = 0; j < response.data.warnings.length; j++ ) {
								Localized.setNotification( {
									type: 'warning',
									message: response.data.warnings[ j ],
								} );
							}
						}
						if ( response.data.errors ) {
							for ( let j = 0; j < response.data.errors.length; j++ ) {
								Localized.setNotification( {
									type: 'error',
									message: response.data.errors[ j ],
								} );
							}
						}
						return response.data;
					}
					Localized.setNotification( {
						type: 'danger',
						message: response.statusText,
					} );
					return response;
				},
				function errorCallback( response ) {
					if ( response.status === 401 ) {
						if ( ! response.config.url.includes( 'tenant' ) ) {
							Admin.Logout();
						} else {
							Tenant.Logout();
						}
					}
					Localized.setNotification( {
						type: 'danger',
						message: response.statusText,
					} );
					return response;
				},
			);
			return promise;
		}

		const service: any = {
			httpGet,
			httpPost,
			httpPut,
			httpDelete,
			getHttpPromise,
			setLocation,
			cleanKey,
			cleanErrorMessages,
			trimExtraSp,
		};

		return service;

		/**
		 * Get all items.
		 *
		 * @param {string} endpoint REST API endpoint
		 * @param {Object} params   Parameters
		 * @param {Object} data     Data
		 * @return {Object} Promise
		 */
		function httpGet( endpoint: string, params: any, data: any ) {
			params = params || {};
			data = data || {};
			const req = { method: 'GET', url: endpoint, params, data };
			return httpPromise( req );
		}

		/**
		 * Create an item.
		 *
		 * @param {string} endpoint REST API endpoint
		 * @param {Object} params   Parameters
		 * @param {Object} data     Data
		 * @return {Object} Promise
		 */
		function httpPost( endpoint: string, params: any, data: any ) {
			params = params || {};
			data = data || {};
			const req = { method: 'POST', url: endpoint, params, data };
			return httpPromise( req );
		}

		/**
		 * Update an existing item.
		 *
		 * @param {string} endpoint REST API endpoint
		 * @param {Object} params   Parameters
		 * @param {Object} data     Data
		 * @return {Object} Promise
		 */
		function httpPut( endpoint: string, params: any, data: any ) {
			params = params || {};
			data = data || {};
			const req = { method: 'PUT', url: endpoint, params, data };
			return httpPromise( req );
		}

		/**
		 * Delete an item.
		 *
		 * @param {string} endpoint REST API endpoint
		 * @param {Object} params   Parameters
		 * @param {Object} data     Data
		 * @return {Object} Promise
		 */
		function httpDelete( endpoint: string, params: any, data: any ) {
			params = params || {};
			data = data || {};
			const req = { method: 'DELETE', url: endpoint, params, data };
			return httpPromise( req );
		}

		/**
		 * @param {Object} req Request
		 * @return {Object} Promise
		 */
		function getHttpPromise( req: any ) {
			return httpPromise( req );
		}

		/**
		 * @param {string}  location     Path
		 * @param {boolean} shouldReturn Should return
		 */
		function setLocation( location: string, shouldReturn: boolean ) {
			if ( ! service.redirecting ) {
				service.redirecting = true;
				if ( shouldReturn === true ) {
					service.ReturnLocation = $location.url();
				}
				$location.path( location );
			}
		}

		/**
		 * @param {string} str Key
		 * @return {string} Cleaned key
		 */
		function cleanKey( str: string ) {
			return str.toLowerCase().replace( /\s+/g, '-' );
		}

		/**
		 * @param {string} str Message
		 * @return {string} Trimmed message
		 */
		function cleanErrorMessages( str: string ) {
			return str.toString().trim();
		}

		/**
		 * @param {string} str String
		 * @return {string} Trimmed string
		 */
		function trimExtraSp( str: string ) {
			return str.replace( /\s+/g, '' );
		}
	}
}() );
