export function analyticsService() {
	const self = this;
	( window as any ).dataLayer = window.dataLayer || [];

	function gtag() {
		dataLayer.push( arguments );
	}

	gtag( 'js', new Date() );

	try {
		if ( localized.settings.analytics.UA.enabled ) {
			self.UAEnabled = true;
			gtag( 'config', localized.settings.analytics.UA.id );
		} else {
			self.UAEnabled = false;
		}
	} catch ( e ) {
		console.warn( e.message );
		self.UAEnabled = false;
	}

	try {
		if ( localized.settings.analytics.AW.enabled ) {
			self.AWEnabled = true;
			gtag( 'config', localized.settings.analytics.AW.id );
		} else {
			self.AWEnabled = false;
		}

		if ( localized.settings.analytics.conversions.phone.enabled ) {
			gtag( 'config', localized.settings.analytics.AW.id + '/' + localized.settings.analytics.conversions.phone.tag, {
				phone_conversion_number: localized.settings.analytics.conversions.phone.number,
			} );
		}
	} catch ( e ) {
		console.warn( e.message );
		self.AWEnabled = false;
	}
	/**
	 * Send any event
	 *
	 * @function
	 * @param {string} action
	 * @param {Object} data
	 */

	self.reportEvent = function( action, data ) {
		if ( self.UAEnabled || self.AWEnabled ) {
			try {
				gtag( 'event', action, data );
			} catch ( e ) {
				console.warn( e.message );
			}
		}
	};
	/**
	 * Add to Cart event
	 *
	 * @function
	 * @param {Object} cart             - Contains the data for sending to analytics for add to cart
	 * @param {float}  cart.added_value - The combined value of all items added to cart
	 * @param {Object} cart.added_items - All of the items added to the cart
	 */

	self.addToCart = function( cart ) {
		try {
			self.reportEvent( 'add_to_cart', {
				currency: 'USD',
				value: cart.added_value,
				items: cart.added_items,
			} );
		} catch ( e ) {
			console.warn( e.message );
		}
	};
	/**
	 * Begin Checkout event
	 *
	 * @function
	 * @param {Object} order
	 * @param {float}  order.order.total_amt - The total value of order before tax/shipping calculations
	 * @param {Object} order.checkout_items  - The line items of the order
	 */

	self.beginCheckout = function( order ) {
		try {
			self.reportEvent( 'begin_checkout', {
				currency: 'USD',
				value: order.order.total_amt,
				items: order.checkout_items,
			} );
		} catch ( e ) {
			console.warn( e.message );
		}
	};
	/**
	 * Login event
	 *
	 * @function
	 */

	self.login = function() {
		try {
			self.reportEvent( 'login', {
				method: 'VendorFuel',
			} );
		} catch ( e ) {
			console.warn( e.message );
		}
	};
	/**
	 * Purchase event
	 *
	 * @function
	 * @param {Object}  order
	 * @param {float}   order.order_info.total_amt
	 * @param {integer} order.order_info.order_id
	 * @param {float}   order.order_info.tax
	 * @param {float}   order.order_info.shipping
	 * @param {Object}  order.purchased_items
	 */

	self.purchase = function( order ) {
		try {
			if ( localized.settings.analytics.UA.enabled ) {
				self.reportEvent( 'purchase', {
					currency: 'USD',
					affiliation: 'VendorFuel online store',
					value: order.order_info.total_amt,
					transaction_id: order.order_info.order_id,
					tax: order.order_info.tax,
					shipping: order.order_info.shipping,
					items: order.purchased_items,
				} );
			}

			if ( localized.settings.analytics.AW.enabled && localized.settings.analytics.conversions.purchase.enabled ) {
				self.reportEvent( 'conversion', {
					send_to: localized.settings.analytics.AW.id + '/' + localized.settings.analytics.conversions.purchase.tag,
					transaction_id: order.order_info.order_id,
					currency: 'USD',
					value: order.order_info.total_amt,
				} );
			}
		} catch ( e ) {
			console.warn( e.message );
		}
	};
	/**
	 * Remove From Cart event
	 *
	 * @function
	 * @param {Object} cart               - Contains the data for sending to analytics for remove from cart
	 * @param {float}  cart.removed_value - The combined value of all items removed from cart
	 * @param {Object} cart.removed_items - All of the items removed from the cart
	 */

	self.removeFromCart = function( cart ) {
		try {
			self.reportEvent( 'remove_from_cart', {
				currency: 'USD',
				value: cart.removed_value,
				items: cart.removed_items,
			} );
		} catch ( e ) {
			console.warn( e.message );
		}
	};
	/**
	 * Search event
	 *
	 * @function
	 * @param {string} term - The term that was searched for
	 */

	self.search = function( term ) {
		try {
			if ( term?.length > 0 ) {
				self.reportEvent( 'search', {
					search_term: term,
				} );
			}
		} catch ( e ) {
			console.warn( e.message );
		}
	};
	/**
	 * Sign Up event
	 *
	 * @function
	 */

	self.signUp = function() {
		try {
			self.reportEvent( 'sign_up', {
				method: 'VendorFuel',
			} );
		} catch ( e ) {
			console.warn( e.message );
		}
	};
	/**
	 * View Item event
	 *
	 * @function
	 * @param {Object} product - The product that was retrieved.
	 */

	self.viewItem = function( product ) {
		try {
			self.reportEvent( 'view_item', {
				items: [ {
					id: product.sku,
					name: product.description,
					list_name: 'Product Details',
					brand: product.manufacturer,
					category: product.cat_title,
					price: product.price,
				} ],
			} );
		} catch ( e ) {
			console.warn( e.message );
		}
	};
	/**
	 * View Search Results event
	 *
	 * @function
	 * @param {string} term     - The term that was searched for
	 * @param {Object} products - The products return from this search.
	 */

	self.viewSearchResults = function( term, products ) {
		try {
			const reportProducts = [];

			for ( let i = 0; i < products.length; i++ ) {
				const product = products[ i ];
				reportProducts[ i ] = {
					id: product.sku,
					name: product.description,
					list_name: 'Search Results',
					brand: product.manufacturer ? product.manufacturer : null,
					category: product.category ? product.category.title : null,
					list_position: i,
					price: product.price,
				};
			}

			self.reportEvent( 'view_search_results', {
				search_term: term,
				items: reportProducts,
			} );
		} catch ( e ) {
			console.warn( e.message );
		}
	};
}
